@@ -1,6 +1,6 @@
 /* locale.c - Miscellaneous internationalization functions. */
 
-/* Copyright (C) 1996-2009,2012 Free Software Foundation, Inc.
+/* Copyright (C) 1996-2009,2012,2016-2021 Free Software Foundation, Inc.
 
    This file is part of GNU Bash, the Bourne Again SHell.
 
@@ -43,8 +43,11 @@
 extern int errno;
 #endif
 
-int locale_utf8locale;	/* unused for now */
+int locale_utf8locale;
 int locale_mb_cur_max;	/* value of MB_CUR_MAX for current locale (LC_CTYPE) */
+int locale_shiftstates = 0;
+
+int singlequote_translations = 0;	/* single-quote output of $"..." */
 
 extern int dump_translatable_strings, dump_po_strings;
 
@@ -65,10 +68,10 @@ static char *lang;
 
 /* Called to reset all of the locale variables to their appropriate values
    if (and only if) LC_ALL has not been assigned a value. */
-static int reset_locale_vars __P((void));
+static int reset_locale_vars PARAMS((void));
 
-static void locale_setblanks __P((void));
-static int locale_isutf8 __P((char *));
+static void locale_setblanks PARAMS((void));
+static int locale_isutf8 PARAMS((char *));
 
 /* Set the value of default_locale and make the current locale the
    system default locale.  This should be called very early in main(). */
@@ -79,11 +82,19 @@ set_default_locale ()
   default_locale = setlocale (LC_ALL, "");
   if (default_locale)
     default_locale = savestring (default_locale);
+#else
+  default_locale = savestring ("C");
 #endif /* HAVE_SETLOCALE */
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 
   locale_mb_cur_max = MB_CUR_MAX;
+  locale_utf8locale = locale_isutf8 (default_locale);
+#if defined (HANDLE_MULTIBYTE)
+  locale_shiftstates = mblen ((char *)NULL, 0);
+#else
+  locale_shiftstates = 0;
+#endif
 }
 
 /* Set default values for LC_CTYPE, LC_COLLATE, LC_MESSAGES, LC_NUMERIC and
@@ -103,6 +114,14 @@ set_default_locale_vars ()
       setlocale (LC_CTYPE, lc_all);
       locale_setblanks ();
       locale_mb_cur_max = MB_CUR_MAX;
+      locale_utf8locale = locale_isutf8 (lc_all);
+
+#    if defined (HANDLE_MULTIBYTE)
+      locale_shiftstates = mblen ((char *)NULL, 0);
+#    else
+      locale_shiftstates = 0;
+#    endif
+
       u32reset ();
     }
 #  endif
@@ -203,6 +222,14 @@ set_locale_var (var, value)
 	}
       locale_setblanks ();
       locale_mb_cur_max = MB_CUR_MAX;
+      /* if LC_ALL == "", reset_locale_vars has already called this */
+      if (*lc_all && x)
+	locale_utf8locale = locale_isutf8 (lc_all);
+#  if defined (HANDLE_MULTIBYTE)
+      locale_shiftstates = mblen ((char *)NULL, 0);
+#  else
+      locale_shiftstates = 0;
+#  endif
       u32reset ();
       return r;
 #else
@@ -219,6 +246,14 @@ set_locale_var (var, value)
 	  x = setlocale (LC_CTYPE, get_locale_var ("LC_CTYPE"));
 	  locale_setblanks ();
 	  locale_mb_cur_max = MB_CUR_MAX;
+	  /* if setlocale() returns NULL, the locale is not changed */
+	  if (x)
+	    locale_utf8locale = locale_isutf8 (x);
+#if defined (HANDLE_MULTIBYTE)
+	  locale_shiftstates = mblen ((char *)NULL, 0);
+#else
+	  locale_shiftstates = 0;
+#endif
 	  u32reset ();
 	}
 #  endif
@@ -309,7 +344,7 @@ get_locale_var (var)
   locale = lc_all;
 
   if (locale == 0 || *locale == 0)
-    locale = get_string_value (var);	/* XXX - mem leak? */
+    locale = get_string_value (var);	/* XXX - no mem leak */
   if (locale == 0 || *locale == 0)
     locale = lang;
   if (locale == 0 || *locale == 0)
@@ -327,15 +362,16 @@ get_locale_var (var)
 static int
 reset_locale_vars ()
 {
-  char *t;
+  char *t, *x;
 #if defined (HAVE_SETLOCALE)
   if (lang == 0 || *lang == '\0')
     maybe_make_export_env ();		/* trust that this will change environment for setlocale */
   if (setlocale (LC_ALL, lang ? lang : "") == 0)
     return 0;
 
+  x = 0;
 #  if defined (LC_CTYPE)
-  t = setlocale (LC_CTYPE, get_locale_var ("LC_CTYPE"));
+  x = setlocale (LC_CTYPE, get_locale_var ("LC_CTYPE"));
 #  endif
 #  if defined (LC_COLLATE)
   t = setlocale (LC_COLLATE, get_locale_var ("LC_COLLATE"));
@@ -352,12 +388,19 @@ reset_locale_vars ()
 
   locale_setblanks ();  
   locale_mb_cur_max = MB_CUR_MAX;
+  if (x)
+    locale_utf8locale = locale_isutf8 (x);
+#  if defined (HANDLE_MULTIBYTE)
+  locale_shiftstates = mblen ((char *)NULL, 0);
+#  else
+  locale_shiftstates = 0;
+#  endif
   u32reset ();
-
 #endif
   return 1;
 }
 
+#if defined (TRANSLATABLE_STRINGS)
 /* Translate the contents of STRING, a $"..." quoted string, according
    to the current locale.  In the `C' or `POSIX' locale, or if gettext()
    is not available, the passed string is returned unchanged.  The
@@ -472,7 +515,7 @@ mk_msgstr (string, foundnlp)
    by the caller.  The length of the translated string is returned in LENP,
    if non-null. */
 char *
-localeexpand (string, start, end, lineno, lenp)
+locale_expand (string, start, end, lineno, lenp)
      char *string;
      int start, end, lineno, *lenp;
 {
@@ -525,6 +568,7 @@ localeexpand (string, start, end, lineno, lenp)
       return (temp);
     }
 }
+#endif
 
 /* Set every character in the <blank> character class to be a shell break
    character for the lexical analyzer when the locale changes. */
@@ -535,7 +579,7 @@ locale_setblanks ()
 
   for (x = 0; x < sh_syntabsiz; x++)
     {
-      if (isblank (x))
+      if (isblank ((unsigned char)x))
 	sh_syntaxtab[x] |= CSHBRK|CBLANK;
       else if (member (x, shell_break_chars))
 	{
@@ -547,17 +591,55 @@ locale_setblanks ()
     }
 }
 
+/* Parse a locale specification
+     language[_territory][.codeset][@modifier][+special][,[sponsor][_revision]]
+   and return TRUE if the codeset is UTF-8 or utf8 */
 static int
 locale_isutf8 (lspec)
      char *lspec;
 {
-  char *cp;
+  char *cp, *encoding;
 
 #if HAVE_LANGINFO_CODESET
   cp = nl_langinfo (CODESET);
   return (STREQ (cp, "UTF-8") || STREQ (cp, "utf8"));
+#elif HAVE_LOCALE_CHARSET
+  cp = locale_charset ();
+  return (STREQ (cp, "UTF-8") || STREQ (cp, "utf8"));
 #else
   /* Take a shot */
-  return (strstr (lspec, "UTF-8") || strstr (lspec, "utf8"));
+  for (cp = lspec; *cp && *cp != '@' && *cp != '+' && *cp != ','; cp++)
+    {
+      if (*cp == '.')
+	{
+	  for (encoding = ++cp; *cp && *cp != '@' && *cp != '+' && *cp != ','; cp++)
+	    ;
+	  /* The encoding (codeset) is the substring between encoding and cp */
+	  if ((cp - encoding == 5 && STREQN (encoding, "UTF-8", 5)) ||
+	      (cp - encoding == 4 && STREQN (encoding, "utf8", 4)))
+	    return 1;
+	  else
+	    return 0;
+	}
+    }
+  return 0;
 #endif
 }
+
+#if defined (HAVE_LOCALECONV)
+int
+locale_decpoint ()
+{
+  struct lconv *lv;
+
+  lv = localeconv ();
+  return (lv && lv->decimal_point && lv->decimal_point[0]) ? lv->decimal_point[0] : '.';
+}
+#else
+#  undef locale_decpoint
+int
+locale_decpoint ()
+{
+  return '.';
+}
+#endif
